@@ -76,7 +76,6 @@ def load_configuration():
     #
     # Logging
     #
-
     CONFIG.add_section('logging')
     CONFIG.set('logging', 'level', getenv('QGSRV_LOGGING_LEVEL', 'DEBUG'))
     CONFIG.set('logging', 'qgis_info', getenv('QGSRV_LOGGING_QGIS_INFO', 'no'))
@@ -115,8 +114,9 @@ def load_configuration():
                getenv2('QGSRV_TRUST_LAYER_METADATA','QGIS_SERVER_TRUST_LAYER_METADATA','no'))
     CONFIG.set('projects.cache', 'disable_getprint'    , 
                getenv2('QGSRV_DISABLE_GETPRINT','QGIS_SERVER_DISABLE_GETPRINT','no'))
-    CONFIG.set('projects.cache', 'disable_owsurls'    , getenv('QGSRV_CACHE_DISABLE_OWSURLS','no'))
-    
+    CONFIG.set('projects.cache', 'disable_owsurls'    , getenv('QGSRV_CACHE_DISABLE_OWSURLS','no')) 
+    CONFIG.set('projects.cache', 'force_etag'         , getenv('QGSRV_CACHE_FORCE_ETAG','no')) 
+
     # 
     CONFIG.add_section('projects.schemes')
 
@@ -195,7 +195,11 @@ def validate_config_path(confname, confid, optional=False):
 def configure_qgis_api( name: str ) -> None:
     """ Configure qgis service environnement variables
     """
-    config = CONFIG[f"api:{name}"]
+    section = f"api:{name}"
+    if not CONFIG.has_section(section):
+        return
+
+    config = CONFIG[section]
     for k,v in config.items():
         LOGGER.debug("configuring qgis api '%s': %s = %s", name, k, v) 
         os.environ[k] = v
